function [timepoints,covariatepoints,haz,h]=khazardcond(time,covariate,delta,plot_out,h,t,x,type,typew,kernel,parallel,tol)
% Kernel estimate of conditional hazard function for right-censored data. Options include two methods for bandwidth selection.
%Input:
% time vector of observed time
% delta vector of censoring indicator. 0 - censored, 1 - uncensored (dead)
% covariate vector of covariate
% plot_out graphical output. Default is true.
% h bandwidth vector of length 2, first element is bandwidth for time and second for covariate or string of method for bandwidth selection ('crossval' (default), 'maxlike'). If missing, h is found using some bandwidth selection method.
% t vector of time points at which estimate is evaluated
% x vector of covariate points at which estimate is evaluated
% type.w Type of weights. Default are Nadaraya-Watson weights.
% type Type of kernel estimate. Possible types are:  'exterior', 'interior' (default).
% kernel kernel function, possible values are: 'epanechnikov' (default), 'gaussian', 'rectangular', 'quartic'.
% parallel  allows parallel computation. Default is false.
% tol  the desired accuracy of optimization algorithm
%Output:
% timepoints vector of time points at which estimate is evaluated
% covariatepoints vector of covariate points at which estimate is evaluated
% haz data frame of time points, hazard function values and bandwidth
% h{bandwidth}

narginchk(3,12)
nargoutchk(3,4)
expectedkernel = {'epanechnikov', 'gaussian', 'rectangular', 'quartic'};
expectedtype = {'exterior', 'interior'};
expectedtypew = {'nw'};
expectedh = {'crossval', 'maxlike'};

if nargin <11
    parallel=false;
end
if nargin <10
    kernel= 'epan';
end
if nargin <9
    type= 'nw';
end
if nargin <8
    type = 'interior';
end
if nargin <7
    x=linspace(min(covariate),max(covariate));
end
if nargin <6
    t=linspace(0,max(time));
end
if nargin <5
    h='crossval';
end

if nargin <4
    plot_out=true;
end

validateattributes(time,{'double'},{'nonnegative'},mfilename,'time',1);
validateattributes(delta,{'double'},{'nonnegative'},mfilename,'delta',3);
validateattributes(covariate,{'double'},{'nonempty'},mfilename,'covariate',2);
validateattributes(plot_out,{'logical'},{'nonempty'},mfilename,'plot_out',4);
validateattributes(h,{'char','double'},{'nonempty'},mfilename,'h',5);
validateattributes(t,{'double'},{'nonnegative','finite'},mfilename,'t',6);
validateattributes(x,{'double'},{'finite'},mfilename,'x',7);
validateattributes(type,{'char'},{'nonempty'},mfilename,'type',8);
validateattributes(typew,{'char'},{'nonempty'},mfilename,'typew',9);
validateattributes(kernel,{'char'},{'nonempty'},mfilename,'kernel',10);
validateattributes(parallel,{'logical'},{'nonempty'},mfilename,'parallel',11);

h_fun=[];
if ischar(h)
    h_fun = validatestring(h,expectedh,mfilename,'h',5);
    h=[];
elseif any(h<=0)
    error('Error. h must have positive elements.')
end
type = validatestring(type,expectedtype,mfilename,'type',8);
typew = validatestring(typew,expectedtypew,mfilename,'typew',9);
kernel = validatestring(kernel,expectedkernel,mfilename,'kernel',10);

if length(time)==length(delta)&&length(time)==length(covariate)
    I=find(isfinite(time)&isfinite(delta)&isfinite(covariate));
    time=time(I);
    covariate=covariate(I);
    delta=delta(I);
    validateattributes(delta,{'double'},{'binary'},mfilename,'delta',2);
else
    error('Error. Time, covariate and delta must have the same length.')
end


if ~isempty(h_fun)
    ax=(max(covariate) - min(covariate)) / length(covariate);
    bx=range(covariate);
    at=(max(times) - min(times)) / length(times);
    bt=max(times);
    
    switch h_fun
        case 'crossval'
            if nargin <12
                tol = 10^(-6);
            end
            validateattributes(tol,{'double'},{'positive','scalar'},mfilename,'tol',12);
            h=evolalg('crossval_haz_cond',50,at,bt,ax,bx,tol,true,time,covariate,delta,type,typew,kernel,parallel);
        case 'maxlike'
            if nargin <12
                tol = 1;
            end
            validateattributes(tol,{'double'},{'positive','scalar'},mfilename,'tol',12);
            h=evolalg('maxlike_haz_cond',50,at,bt,ax,bx,tol,false,time,covariate,delta,type,typew,kernel,parallel);
    end
end

haz=hazard_cond(t,x,Y,X,d,h,kernel,type,typew);
timepoints=t;
covariatepoints=x;
if plot_out
    figure
    surf(timepoints,covariatepoints,haz')
    title(['Kernel estimation of hazard function, ht=', num2str( h(1) ),', hx=', num2str( h(2))])
    xlabel('Time')
    ylabel('Covariate')
    zlabel('Hazard Rate')
      
end


end