function [timepoints,haz,h]=khazard(time,delta,plot_out,h,t,type,kernel,parallel,tol)
% Kernel estimate of (unconditional) hazard function for right-censored data. Options include two methods for bandwidth selection.
%Input:
% time vector of observed times
% delta vector of censoring indicator. 0 - censored, 1 - uncensored (dead)
% plot_out graphical output. Default is true.
% h bandwidth (scalar or vector) or string of method for bandwidth selection ('crossval' (default), 'maxlike'). If missing, h is found using some bandwidth selection method.
% t vector of time points at which estimate is evaluated
% kernel kernel function, possible values are: 'epanechnikov' (default), 'gaussian', 'rectangular', 'quartic'.
% type Type of kernel estimate. Possible types are:  'exterior', 'interior' (default).
% parallel  allows parallel computation. Default is false.
% tol  the desired accuracy of optimization algorithm
%Output:
% timepoints vector of time points at which estimate is evaluated
% haz vector of hazard function values
% h used bandwidth

w=which('khazard.m');
w(end-9:end)=[];
addpath(w);
addpath([w,'/base']);
addpath([w,'/unconditional']);

narginchk(2,9)
nargoutchk(2,3)
expectedkernel = {'epanechnikov', 'gaussian', 'rectangular', 'quartic'};
expectedtype = {'exterior', 'interior'};
expectedh = {'crossval', 'maxlike'};


if nargin <8
    parallel=false;
end
if nargin <7
    kernel= 'epan';
end
if nargin <6
    type = 'interior';
end
if nargin <5
    t=linspace(0,max(time));
end
if nargin <4
    h='crossval';
end

if nargin <3
    plot_out=true;
end

validateattributes(time,{'double'},{'nonnegative'},mfilename,'time',1);
validateattributes(delta,{'double'},{'nonnegative'},mfilename,'delta',2);
validateattributes(plot_out,{'logical'},{'nonempty'},mfilename,'plot_out',3);
validateattributes(h,{'char','double'},{'nonempty'},mfilename,'h',4);
validateattributes(t,{'double'},{'nonnegative','finite'},mfilename,'t',5);
validateattributes(type,{'char'},{'nonempty'},mfilename,'type',6);
validateattributes(kernel,{'char'},{'nonempty'},mfilename,'kernel',7);
validateattributes(parallel,{'logical'},{'nonempty'},mfilename,'parallel',8);

h_fun=[];
if ischar(h)
    h_fun = validatestring(h,expectedh,mfilename,'h',4);
    h=[];
elseif any(h<=0)
    error('Error. h must have positive elements.')
end
type = validatestring(type,expectedtype,mfilename,'type',6);
kernel = validatestring(kernel,expectedkernel,mfilename,'kernel',7);

if length(time)==length(delta)
    I=find(isfinite(time)&isfinite(delta));
    time=time(I);
    delta=delta(I);
    validateattributes(delta,{'double'},{'binary'},mfilename,'delta',2);
else
    error('Error. Time and delta must have the same length.')
end


if ~isempty(h_fun)
    a = (max(time) - min(time)) / length(time);
    b = max(time);
    switch h_fun
        case 'crossval'
            if nargin <9
                tol = 10^(-6);
            end
            validateattributes(tol,{'double'},{'positive','scalar'},mfilename,'tol',9);
            h=opt_osci('crossval_haz',a,b,tol,4,true,time,delta,type,kernel,parallel);
        case 'maxlike'
            if nargin <9
                tol = 1;
            end
            validateattributes(tol,{'double'},{'positive','scalar'},mfilename,'tol',9);
            h=opt_osci('maxlike_haz',a,b,tol,4,false,time,delta,type,kernel,parallel);
    end
end


if length(h)==1
    haz=hazard(t,time,delta,h,kernel,type);
    timepoints=t;
    if plot_out
        figure
        plot(timepoints,haz)
        title(['Kernel estimation of hazard function, h=', num2str( h )])
        xlabel('Time')
        ylabel('Hazard Rate')
    end
else
    timepoints=t;
    haz=zeros(length(timepoints),length(h));
    for i=1:length(h)
        haz(:,i)=hazard(t,time,delta,h(i),kernel,type);
    end
    
    
    if plot_out
        figure
        surf(timepoints,h,haz')
        title('Kernel estimation of hazard function')
        xlabel('Time')
        ylabel('Bandwidths')
        zlabel('Hazard Rate')
    end
end

end



